/*! \file enc28j60.h \brief Microchip ENC28J60 Ethernet Interface Driver. */
//*****************************************************************************
//
// File Name	: 'enc28j60.h'
// Title		: Microchip ENC28J60 Ethernet Interface Driver
// Author		: Pascal Stang (c)2005
// Created		: 9/22/2005
// Revised		: 9/22/2005
// Version		: 0.1
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
///	\ingroup network
///	\defgroup enc28j60 Microchip ENC28J60 Ethernet Interface Driver (enc28j60.c)
///	\code #include "net/enc28j60.h" \endcode
///	\par Overview
///		This driver provides initialization and transmit/receive
///	functions for the Microchip ENC28J60 10Mb Ethernet Controller and PHY.
/// This chip is novel in that it is a full MAC+PHY interface all in a 28-pin
/// chip, using an SPI interface to the host processor.
///
//
//*****************************************************************************

#include "avr/io.h"
#include <util/delay.h>

#include "enc28j60.h"

#define delay_us(x) for(unsigned long a=0;a<(x);a++){asm("nop");};

/*
#ifdef SPDR0
	#define SPDR	SPDR0
	#define SPCR	SPCR0
	#define SPSR	SPSR0

	#define SPIF	SPIF0
	#define MSTR	MSTR0
	#define CPOL	CPOL0
	#define DORD	DORD0
	#define SPR0	SPR00
	#define SPR1	SPR01
	#define SPI2X	SPI2X0
	#define SPE		SPE0
#endif
*/

unsigned char Enc28j60Bank;
unsigned int NextPacketPtr;

unsigned char mymac[6];
unsigned char myip[4];

unsigned char SoftwareSPI(unsigned char in)
{
	unsigned char data = 0;
	for (unsigned i = 0; i < 8; ++i)
	{
		// write
		if (in & 128)
			ENC28J60_SPI_PORT |= (1 << ENC28J60_SPI_MOSI);
		else
			ENC28J60_SPI_PORT &= ~(1 << ENC28J60_SPI_MOSI);
		in <<= 1;

		_delay_us(3);
		// read
		data <<= 1;
		if (ENC28J60_SPI_PIN & (1 << ENC28J60_SPI_MISO))
			data |= 1;

		// clock
		ENC28J60_SPI_PORT |= (1 << ENC28J60_SPI_SCK);
		// clock low
		ENC28J60_SPI_PORT &= ~(1 << ENC28J60_SPI_SCK);
	}
	return data;
}
unsigned char enc28j60ReadOp(unsigned char op, unsigned char address)
{
	unsigned char data;
   
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	//delay_us(200);
	
	// issue read command
	//SPDR = op | (address & ADDR_MASK);
	//while(!(SPSR & (1<<SPIF)));
	SoftwareSPI(op | (address & ADDR_MASK));
	// read data
	//SPDR = 0x00;
	//while(!(SPSR & (1<<SPIF)));
	data = SoftwareSPI(0);
	// do dummy read if needed
	if(address & 0x80)
	{
		//SPDR = 0x00;
		//while(!(SPSR & (1<<SPIF)));
		data = SoftwareSPI(0);
	}
	//data = SPDR;
	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);

	return data;
}

void enc28j60WriteOp(unsigned char op, unsigned char address, unsigned char data)
{
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	//delay_us(200);
	
	// issue write command
	//SPDR = op | (address & ADDR_MASK);
	//while(!(SPSR & (1<<SPIF)));
	SoftwareSPI(op | (address & ADDR_MASK));
	// write data
	//SPDR = data;
	//while(!(SPSR & (1<<SPIF)));
	SoftwareSPI(data);

	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60ReadBuffer(unsigned int len, unsigned char* data)
{
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	//delay_us(200);
	
	// issue read command
	//SPDR = ENC28J60_READ_BUF_MEM;
	//while(!(SPSR & (1<<SPIF)));
	SoftwareSPI(ENC28J60_READ_BUF_MEM);
	while(len--)
	{
		// read data
		//SPDR = 0x00;
		//while(!(SPSR & (1<<SPIF)));
		//*data++ = SPDR;
		*data++ = SoftwareSPI(0);
	}	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60WriteBuffer(unsigned int len, unsigned char* data)
{
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	//delay_us(200);
	
	// issue write command
	//SPDR = ENC28J60_WRITE_BUF_MEM;
	//while(!(SPSR & (1<<SPIF)));
	SoftwareSPI(ENC28J60_WRITE_BUF_MEM);
	while(len--)
	{
		// write data
		//SPDR = *data++;
		//while(!(SPSR & (1<<SPIF)));
		SoftwareSPI(*data++);
	}	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60SetBank(unsigned char address)
{
	// set the bank (if needed)
	if((address & BANK_MASK) != Enc28j60Bank)
	{
		// set the bank
		enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, (ECON1_BSEL1|ECON1_BSEL0));
		enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, (address & BANK_MASK)>>5);
		Enc28j60Bank = (address & BANK_MASK);
	}
}

unsigned char enc28j60Read(unsigned char address)
{
	// set the bank
	enc28j60SetBank(address);
	// do the read
	return enc28j60ReadOp(ENC28J60_READ_CTRL_REG, address);
}

void enc28j60Write(unsigned char address, unsigned char data)
{
	// set the bank
	enc28j60SetBank(address);
	// do the write
	enc28j60WriteOp(ENC28J60_WRITE_CTRL_REG, address, data);
}

unsigned int enc28j60PhyRead(unsigned char address)
{
	unsigned int data;

	// Set the right address and start the register read operation
	enc28j60Write(MIREGADR, address);
	enc28j60Write(MICMD, MICMD_MIIRD);

	// wait until the PHY read completes
	while(enc28j60Read(MISTAT) & MISTAT_BUSY);

	// quit reading
	enc28j60Write(MICMD, 0x00);
	
	// get data value
	data  = enc28j60Read(MIRDL);
	data |= enc28j60Read(MIRDH) << 8;
	// return the data
	return data;
}

void enc28j60PhyWrite(unsigned char address, unsigned int data)
{
	// set the PHY register address
	enc28j60Write(MIREGADR, address);
	
	// write the PHY data
	enc28j60Write(MIWRL, data);	
	enc28j60Write(MIWRH, data >> 8);

	// wait until the PHY write completes
	while(enc28j60Read(MISTAT) & MISTAT_BUSY);
}

void enc28j60Init(void)
{
	// initialize I/O
	ENC28J60_CONTROL_DDR |= (1<<ENC28J60_CONTROL_CS);
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);

	// setup SPI I/O pins
	ENC28J60_SPI_PORT |= (1<<ENC28J60_SPI_SCK);	// set SCK hi
	ENC28J60_SPI_DDR |= (1<<ENC28J60_SPI_SCK);	// set SCK as output
	ENC28J60_SPI_DDR &= ~(1<<ENC28J60_SPI_MISO);	// set MISO as input
	ENC28J60_SPI_DDR |= (1<<ENC28J60_SPI_MOSI);	// set MOSI as output
	//ENC28J60_SPI_DDR |= (1<<ENC28J60_SPI_SS);		// SS must be output for Master mode to work
	// initialize SPI interface
	// master mode
	//SPCR |= (1<<MSTR);

	// switch to f/4 2X = f/2 bitrate
	//SPCR|= (1<<SPR0);
	//SPCR |= (1<<SPR1);
	//SPSR |= (1<<SPI2X);
	// enable SPI
	//SPCR |= (1<<SPE);

	// perform system reset
	enc28j60WriteOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET);
	// check CLKRDY bit to see if reset is complete
	delay_us(100000);
	while(!(enc28j60Read(ESTAT) & ESTAT_CLKRDY));

	// do bank 0 stuff
	// initialize receive buffer
	// 16-bit transfers, must write low byte first
	// set receive buffer start address
	/*NextPacketPtr = RXSTART_INIT;
	enc28j60Write(ERXSTL, RXSTART_INIT&0xFF);
	enc28j60Write(ERXSTH, RXSTART_INIT>>8);
	// set receive pointer address
	enc28j60Write(ERXRDPTL, RXSTART_INIT&0xFF);
	enc28j60Write(ERXRDPTH, RXSTART_INIT>>8);
	// set receive buffer end
	// ERXND defaults to 0x1FFF (end of ram)
	enc28j60Write(ERXNDL, RXSTOP_INIT&0xFF);
	enc28j60Write(ERXNDH, RXSTOP_INIT>>8);
	// set transmit buffer start
	// ETXST defaults to 0x0000 (beginnging of ram)
	enc28j60Write(ETXSTL, TXSTART_INIT&0xFF);
	enc28j60Write(ETXSTH, TXSTART_INIT>>8);*/

	// alloc receivebuffer at 0000 because of a silicon bug
	NextPacketPtr = RXSTART_INIT;
	// set receive buffer start address
	enc28j60Write(ERXSTL, RXSTART_INIT & 0xff);
	enc28j60Write(ERXSTH, RXSTART_INIT >> 8);
	// set receive buffer end address
	enc28j60Write(ERXNDL, RXSTOP_INIT & 0xff);
	enc28j60Write(ERXNDH, RXSTOP_INIT >> 8);
	// set receive buffer read pointer
	enc28j60Write(ERXRDPTL, RXSTART_INIT & 0xff);
	enc28j60Write(ERXRDPTH, RXSTART_INIT >> 8);
	// set transmit buffer start address
	enc28j60Write(ETXSTL, TXSTART_INIT & 0xff);
	enc28j60Write(ETXSTH, TXSTART_INIT >> 8);
	// set transmit buffer end address
	enc28j60Write(ETXNDL, TXSTOP_INIT & 0xff);
	enc28j60Write(ETXNDH, TXSTOP_INIT >> 8);

	// do bank 2 stuff
	// enable MAC receive
	enc28j60Write(MACON1, MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);
	// bring MAC out of reset
	enc28j60Write(MACON2, 0x00);
	// enable automatic padding and CRC operations
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, MACON3, MACON3_PADCFG0 | MACON3_TXCRCEN | MACON3_FRMLNEN);// | MACON3_FULDPX);

	// set inter-frame gap (non-back-to-back)
	enc28j60Write(MAIPGL, 0x12);
	enc28j60Write(MAIPGH, 0x0C);
	// set inter-frame gap (back-to-back)
	enc28j60Write(MABBIPG, 0x12);
	// Set the maximum packet size which the controller will accept
	enc28j60Write(MAMXFLL, MAX_FRAMELEN&0xFF);	
	enc28j60Write(MAMXFLH, MAX_FRAMELEN>>8);

	// do bank 3 stuff
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(MAADR5, mymac[0]);
	enc28j60Write(MAADR4, mymac[1]);
	enc28j60Write(MAADR3, mymac[2]);
	enc28j60Write(MAADR2, mymac[3]);
	enc28j60Write(MAADR1, mymac[4]);
	enc28j60Write(MAADR0, mymac[5]);

	// no loopback of transmitted frames
	enc28j60PhyWrite(PHCON2, PHCON2_HDLDIS);
	//enc28j60PhyWrite(PHCON1, PHSTAT1_PFDPX);

	// switch to bank 0
	enc28j60SetBank(ECON1);
	// enable interrutps
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, EIE, EIE_INTIE | EIE_PKTIE);
	// enable packet reception
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN);
}

void enc28j60PacketSend(unsigned int len, unsigned char* packet)
{
	// ENC28J60 Rev. B7 Silicon Errata workaround #10,
    // Internal transmit logic may stall in half duplex mode
    // on excessive collisions, a late collision or excessive deferrals
    // In this case the Transmit Error Interrupt Flag is set. Check it
    if (!(enc28j60Read(EIR) & EIR_TXERIF))
    {   // Reset transmit logic
    	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRST);
        enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRST);
        enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, EIR, EIR_TXERIF);
    } 
	// Set the write pointer to start of transmit buffer area
	enc28j60Write(EWRPTL, TXSTART_INIT && 0xff);
	enc28j60Write(EWRPTH, TXSTART_INIT >> 8);
	// Set the TXND pointer to correspond to the packet size given
	enc28j60Write(ETXNDL, (TXSTART_INIT+len) & 0xff);
	enc28j60Write(ETXNDH, (TXSTART_INIT+len) >> 8);

	// write per-packet control byte
	enc28j60WriteOp(ENC28J60_WRITE_BUF_MEM, 0, 0x00);

	// copy the packet into the transmit buffer
	enc28j60WriteBuffer(len, packet);
	
	// send the contents of the transmit buffer onto the network
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRTS);
}

unsigned int enc28j60PacketReceive(unsigned int maxlen, unsigned char* packet)
{
	unsigned int rxstat;
	unsigned int len;

	// check if a packet has been received and buffered
	if (!enc28j60Read(EPKTCNT))
		return 0;

	// Set the read pointer to the start of the received packet
	enc28j60Write(ERDPTL, (NextPacketPtr));
	enc28j60Write(ERDPTH, (NextPacketPtr)>>8);
	// read the next packet pointer
	NextPacketPtr  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	NextPacketPtr |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the packet length
	len  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	len |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the receive status
	rxstat  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	rxstat |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;

	// limit retrieve length
	// (we reduce the MAC-reported length by 4 to remove the CRC)
	len = MIN(len, maxlen);

	// copy the packet from the receive buffer
	enc28j60ReadBuffer(len, packet);

	// Move the RX read pointer to the start of the next received packet
	// This frees the memory we just read out
	enc28j60Write(ERXRDPTL, (NextPacketPtr));
	enc28j60Write(ERXRDPTH, (NextPacketPtr)>>8);

	// decrement the packet counter indicate we are done with this packet
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);

	return len;
}




